-- mission.lua 
--
--      Simulator Scripting Engine       Scripting Engine
--      :
-- 1.       
-- 2.       
-- 3.     :
-- 	UnitsGroup.holdOn(self) -  
-- 	UnitsGroup.holdOnForAWhile(self) -     
--  UnitsGroup.moveOn(self) -  
-- 	UnitsGroup.atEase(self),  UnitsGroup.stayAlert(self) -  ,   
-- 	UnitsGroup.shootBack(self), UnitsGroup.fireAtWill(self) -   
--   ..
-- 4.       .
-- 5.     (   ,   )
-- 	            
--  ... 
--      (     Simulator Scripting Engine      
--   )
--              Simulator Scripting Engine
--      , .
--          ,    
--       do script file c   . 

--  : 24.10.2012
-- :  .. (BAntDit)

mission = {
	["model"] = {
		["flags"] = {},
		["phases"] = {},
		["groups"] = {},
		["units"] = {}
	},
	["controller"] = {
	}
}

Factory = {
	setBaseClass = function (class, baseClass) 
		setmetatable(class, {__index = baseClass});
	end, 
	create = function (class, parameters) 
		local newObject = {}
		setmetatable(newObject, {__index = class});
		
		if (nil ~= newObject.construct) then
			newObject:construct(parameters);
		end
		
		return newObject;
	end
}

-- tasks:
GroupTask = {
}

function GroupTask:construct(parameters) 
	self.functionToCall = parameters.functionToCall;
	self.argument = parameters.functionArgument;
	self.id = timer.scheduleFunction(self.functionToCall, self.argument, parameters.startTime);
end;

function GroupTask:Stop()
	timer.removeFunction(self.id);
end;

function GroupTask:Resum(newTime)
	self.id = timer.scheduleFunction(self.functionToCall, self.argument, newTime);
end;

function GroupTask:ResumWithArgument(argument, newTime)
	self.argument = argument;
	self.id = timer.scheduleFunction(self.functionToCall, self.argument, newTime);
end;

function GroupTask:StartImmediately()
	timer.setFunctionTime(self.id, (timer.getTime() + 1));
end;

function GroupTask:StartAfter(delay)
	timer.setFunctionTime(self.id, (timer.getTime() + delay));
end;

function GroupTask:GetId() return self.id; end;

-- events:
BaseEvent = {
}

-- mission phase description:

MissionPhaseDescription = {
};

function MissionPhaseDescription:construct(parameters)
	self.before = parameters.before;
	self.during = parameters.during;
	self.after = parameters.after;
	self.task = parameters.task;
end;

function MissionPhaseDescription:getBefore() return self.before; end;
function MissionPhaseDescription:getDuring() return self.during; end;
function MissionPhaseDescription:getAfter() return self.after; end;
function MissionPhaseDescription:getTask() return self.task; end;

function MissionPhaseDescription:setBefore(_before) self.before = _before; end;
function MissionPhaseDescription:setDuring(_during) self.during = _during; end;
function MissionPhaseDescription:setAfter(_after) self.after = _after; end;
function MissionPhaseDescription:setTask(_task) self.task = _task; end;

-- misson phases
MissionPhase = {
};

function MissionPhase:construct(parameters)
	self.isPhaseActivated = false;
	self.hasPhaseBegun = false;
	self.hasPahseDone = false;
	self.onActivate = Factory.create(BaseEvent, {});
	self.onDeactivate = Factory.create(BaseEvent, {});
	self.description = Factory.create(MissionPhaseDescription, parameters.description);
	self.tasks = {};
end;

function MissionPhase:addOnDeactivateEventHandler(eventHandler)
	local _nextEventHandlerIndex = (#self.onDeactivate + 1);
	self.onDeactivate[_nextEventHandlerIndex] = eventHandler;
end;

function MissionPhase:addOnActivateEventHandler(eventHandler)
	local _nextEventHandlerIndex = (#self.onActivate + 1);
	self.onActivate[_nextEventHandlerIndex] = eventHandler;
end;

function MissionPhase:deactivate(eventArgs)
	self.isPhaseActivated = false;
	self.hasPahseDone = true;

	for _eventHandlerIndex = 1, #self.onDeactivate do
		self.onDeactivate[_eventHandlerIndex](self, eventArgs);
	end
	
	for _taskIndex = 1, #self.tasks do
		self.task[_taskIndex]:Stop();
	end
end;

function MissionPhase:activate(eventArgs)
	self.isPhaseActivated = true;
	self.hasPhaseBegun = true;
	
	for _eventHandlerIndex = 1, #self.onActivate do
		self.onActivate[_eventHandlerIndex](self, eventArgs);
	end
	
	for _taskIndex = 1, #self.tasks do
		self.task[_taskIndex]:StartImmediately();
	end
end;

function MissionPhase:addTask(_functionToCall, _argument)
	local _nextTaskIndex = (#self.tasks + 1);
	self.tasks[_nextTaskIndex] = Factory.create(GroupTask, { functionToCall = _functionToCall, functionArgument = _argument, startTime = 999999 });
end

function MissionPhase:isActivated() return self.isPhaseActivated; end;

function MissionPhase:getDescription() 
	if ((false == self.hasPhaseBegun) and (false == self.hasPahseDone)) then
		return self.description:getBefore(), nil;
	else 
		if ((true == self.hasPhaseBegun) and (false == self.hasPahseDone)) then 
			return self.description:getDuring(), self.description:getTask(); 
		end
	end
	
	return self.description:getAfter(), nil;
end;

function MissionPhase:getDescriptionBefore() return self.description:getBefore(); end;
function MissionPhase:getDescriptionDuring() return self.description:getDuring(); end;
function MissionPhase:getDescriptionAfter() return self.description:getAfter(); end;
function MissionPhase:getDescriptionTask() return self.description:getTask(); end;

function MissionPhase:setDescriptionBefore(_description) self.description:setBefore(_description); end;
function MissionPhase:setDescriptionDuring(_description) self.description:setDuring(_description); end;
function MissionPhase:setDescriptionAfter(_description) self.description:setAfter(_description); end;
function MissionPhase:setDescriptionTask(_description) self.description:setTask(_description); end;

function MissionPhase:begun() return self.hasPhaseBegun; end;

function MissionPhase:done() return self.hasPahseDone; end;

-- unit's wrapper:
UnitWrapper = {
}

function UnitWrapper:construct(parameters) 
	self.dcsWorldUnit = parameters.newUnit;
	self.unitName = parameters.newUnitName;
	self.onHit = Factory.create(BaseEvent, {});
	self.onShot = Factory.create(BaseEvent, {});
	self.onDead = Factory.create(BaseEvent, {});
	self.onPlayerEnterUnit = Factory.create(BaseEvent, {});
	self.onPlayerLeaveUnit = Factory.create(BaseEvent, {});
	self.onTakeOff = Factory.create(BaseEvent, {});
end;

function UnitWrapper:getDCSWorldUnit() return self.dcsWorldUnit; end;

function UnitWrapper:getUnitName() return self.unitName; end;

function UnitWrapper:addOnTakeOffEventHandler(eventHandler)
	self.onTakeOff[eventHandler] = eventHandler;
end;

function UnitWrapper:removeOnTakeOffEventHandler(eventHandler)
	self.onTakeOff[eventHandler] = nil;
end;

function UnitWrapper:addOnPlayerEnterUnit(eventHandler)
	self.onPlayerEnterUnit[eventHandler] = eventHandler;
end;

function UnitWrapper:removeOnPlayerEnterUnit(eventHandler)
	self.onPlayerEnterUnit[eventHandler] = nil;
end;

function UnitWrapper:addOnPlayerLeaveUnit(eventHandler)
	self.onPlayerLeaveUnit[eventHandler] = eventHandler;
end;

function UnitWrapper:removeOnPlayerLeaveUnit(eventHandler)
	self.onPlayerLeaveUnit[eventHandler] = nil;
end;

function UnitWrapper:addOnHitEventHandler(eventHandler)
	self.onHit[eventHandler] = eventHandler;
end;

function UnitWrapper:removeOnHitEventHandler(eventHandler)
	self.onHit[eventHandler] = nil;
end;

function UnitWrapper:addOnShotEventHandler(eventHandler)
	self.onShot[eventHandler] = eventHandler;
end;

function UnitWrapper:removeOnShotEventHandler(eventHandler)
	self.onShot[eventHandler] = nil;
end;

function UnitWrapper:addOnDeadEventHandler(eventHandler)
	self.onDead[eventHandler] = eventHandler;
end;

function UnitWrapper:removeOnDeadEventHandler(eventHandler)
	self.onDead[eventHandler] = nil;
end;

function UnitWrapper:hit(eventArgs)
	for _onHitEventHandlerKey, _onHitEventHandler in pairs(self.onHit) do
		_onHitEventHandler(self, eventArgs);
	end
end;

function UnitWrapper:shot(eventArgs)
	for _onShotEventHandlerKey, _onShotEventHandler in pairs(self.onShot) do
		_onShotEventHandler(self, eventArgs);
	end
end;

function UnitWrapper:dead(eventArgs)
	for _onDeadEventHandlerKey, _onDeadEventHandler in pairs(self.onDead) do
		_onDeadEventHandler(self, eventArgs);
	end
end;

function UnitWrapper:playerEnterUnit(eventArgs)
	for _onPlayerEnterUnitEventHandlerKey, _onPlayerEnterUnitEventHandler in pairs(self.onPlayerEnterUnit) do
		_onPlayerEnterUnitEventHandler(self, eventArgs);
	end
end;

function UnitWrapper:playerLeaveUnit(eventArgs)
	for _onPlayerLeaveUnitEventHandlerKey, _onPlayerLeaveUnitEventHandler in pairs(self.onPlayerLeaveUnit) do
		_onPlayerLeaveUnitEventHandler(self, eventArgs);
	end
end;

function UnitWrapper:takeOff(eventArgs)
	-- trigger.action.outText('debug msg: raised event S_EVENT_TAKEOFF for unit ' .. self.dcsWorldUnit:getName() .. ' (Id number: ' .. self.dcsWorldUnit:getID() .. ' )', 10);
	for _onTakeOffEventHandlerKey, _onTakeOffEventHandler in pairs(self.onTakeOff) do
		_onTakeOffEventHandler(self, eventArgs);
	end
end;

-- group's wrapper:
UnitsGroup = {
	["tasks"] = {}
}

function UnitsGroup:init(eventArgs) 
	for _onInitEventHandlerKey, _onInitEventHandler in pairs(self.onInit) do
		_onInitEventHandler(self, eventArgs);
	end
end;

function UnitsGroup:hit(eventArgs)
	for _onHitEventHandlerKey, _onHitEventHandler in pairs(self.onHit) do
		_onHitEventHandler(self, eventArgs);
	end
end;

function UnitsGroup:addTask(taskName, _functionToCall, _argument, _time)
	self.tasks[taskName] = Factory.create(GroupTask, { functionToCall = _functionToCall, functionArgument = _argument, startTime = _time });
end;

function UnitsGroup:setDCSWorldGroup(value) self.dcsWorldGroup = value; end;

function UnitsGroup:construct(parameters) 
	self.dcsWorldGroup = parameters.newGroup;
	self.onInit = Factory.create(BaseEvent, {});
	self.onHit = Factory.create(BaseEvent, {});
	self.onShot = Factory.create(BaseEvent, {});
	self.onDead = Factory.create(BaseEvent, {});
	self.onPlayerEnterUnit = Factory.create(BaseEvent, {});
	self.onPlayerLeaveUnit = Factory.create(BaseEvent, {});
	self.groupHasStopped = false;
end;

function UnitsGroup:getDCSWorldGroup() return self.dcsWorldGroup; end;

function UnitsGroup:isClearToMoveOn() 
	if (false == self.groupHasStopped) then
		return true;
	end;
	
	return false; 
end;

function UnitsGroup:holdOn()
	local _controller = self.dcsWorldGroup:getController();
	
	self.groupHasStopped = true;
	
	_controller:setTask({
		id = 'ControlledTask',
		params = { 
			task = { id = 'Hold', params = {} }, 
			stopCondition = { condition = 'mission.model.groups.' .. self.dcsWorldGroup:getName() .. ':isClearToMoveOn();' } 
		}
	});
end;

function UnitsGroup:holdOnForAWhile(_delay)
	local _controller = self.dcsWorldGroup:getController();
	
	self.groupHasStopped = true;
	
	_controller:setTask({
		id = 'ControlledTask',
		params = { 
			task = { id = 'Hold', params = {} }, 
			stopCondition = { 
				duration = _delay,
				condition = 'mission.model.groups.' .. self.dcsWorldGroup:getName() .. ':isClearToMoveOn();' }
		}
	});
end;

function UnitsGroup:moveOn()
	self.groupHasStopped = false;
end;

function UnitsGroup:setInvisible(_invisible)
	local _controller = self.dcsWorldGroup:getController();
	_controller:setCommand({ id = 'SetInvisible', params = { value = _invisible }});
end;

function UnitsGroup:setImmortal(_immortal)
	local _controller = self.dcsWorldGroup:getController();
	_controller:setCommand({ id = 'SetImmortal', params = { value = _immortal }});
end;

function UnitsGroup:atEase()
	local _controller = self.dcsWorldGroup:getController();
	_controller:setOption(9, 1);
end;

function UnitsGroup:stayAlert()
	local _controller = self.dcsWorldGroup:getController();
	_controller:setOption(9, 0);
end;

function UnitsGroup:danger()
	local _controller = self.dcsWorldGroup:getController();
	_controller:setOption(9, 2);
end;

function UnitsGroup:holdFire()
	local _controller = self.dcsWorldGroup:getController();
	_controller:setOption(0, 4);
end;

function UnitsGroup:shootBack()
	local _controller = self.dcsWorldGroup:getController();
	_controller:setOption(0, 3);
end;

function UnitsGroup:engageAtWill()
	local _controller = self.dcsWorldGroup:getController();
	_controller:setOption(0, 0);
end;

function UnitsGroup:addOnPlayerEnterUnit(eventHandler)
	self.onPlayerEnterUnit[eventHandler] = eventHandler;
end;

function UnitsGroup:removeOnPlayerEnterUnit(eventHandler)
	self.onPlayerEnterUnit[eventHandler] = nil;
end;

function UnitsGroup:addOnPlayerLeaveUnit(eventHandler)
	self.onPlayerLeaveUnit[eventHandler] = eventHandler;
end;

function UnitsGroup:removeOnPlayerLeaveUnit(eventHandler)
	self.onPlayerLeaveUnit[eventHandler] = nil;
end;

function UnitsGroup:addOnInitEventHandler(eventHandler) 
	self.onInit[eventHandler] = eventHandler;
end;

function UnitsGroup:removeOnInitEventHandler(eventHandler)
	self.onInit[eventHandler] = nil;
end;

function UnitsGroup:addOnHitEventHandler(eventHandler)
	self.onHit[eventHandler] = eventHandler;
end;

function UnitsGroup:removeOnHitEventHandler(eventHandler)
	self.onHit[eventHandler] = nil;
end;

function UnitsGroup:addOnShotEventHandler(eventHandler)
	self.onShot[eventHandler] = eventHandler;
end;

function UnitsGroup:removeOnShotEventHandler(eventHandler)
	self.onShot[eventHandler] = nil;
end;

function UnitsGroup:addOnDeadEventHandler(eventHandler)
	self.onDead[eventHandler] = eventHandler;
end;

function UnitsGroup:removeOnDeadEventHandler(eventHandler)
	self.onDead[eventHandler] = nil;
end;

function UnitsGroup:shot(eventArgs)
	for _onShotEventHandlerKey, _onShotEventHandler in pairs(self.onShot) do
		_onShotEventHandler(self, eventArgs);
	end
end;

function UnitsGroup:dead(eventArgs)
	for _onDeadEventHandlerKey, _onDeadEventHandler in pairs(self.onDead) do
		_onDeadEventHandler(self, eventArgs);
	end
end;

function UnitsGroup:playerEnterUnit(eventArgs)
	for _onPlayerEnterUnitEventHandlerKey, _onPlayerEnterUnitEventHandler in pairs(self.onPlayerEnterUnit) do
		_onPlayerEnterUnitEventHandler(self, eventArgs);
	end
end;

function UnitsGroup:playerLeaveUnit(eventArgs)
	for _onPlayerLeaveUnitEventHandlerKey, _onPlayerLeaveUnitEventHandler in pairs(self.onPlayerLeaveUnit) do
		_onPlayerLeaveUnitEventHandler(self, eventArgs);
	end
end;

-- controller:
function mission.controller:start() 
	for _group_key, _group in pairs(mission.model.groups) do
		_group:init({});
	end
end;

function mission.controller:beginPhase(phaseName)
	if (nil ~= mission.model.phases[phaseName]) then 
		mission.model.phases[phaseName]:activate({});
	else
		trigger.action.outText("error: could not find phase with name: [" .. phaseName .. "]", 10);
	end
end;

function mission.controller:endPhase(phaseName) 
	if (nil ~= mission.model.phases[phaseName]) then 
		mission.model.phases[phaseName]:deactivate({});
	else
		trigger.action.outText("error: could not find phase with name: [" .. phaseName .. "]", 10);
	end
end;

function mission.model:addUnit(unitName)	
	self.units[unitName] = Factory.create(UnitWrapper, { newUnit = Unit.getByName(unitName), newUnitName = unitName });
end;

function mission.model:addGroup(groupName)
	parameters = {};
	
	parameters.newGroup = Group.getByName(groupName);

	if (nil ~= parameters.newGroup) then
		self.groups[groupName] = Factory.create(UnitsGroup, parameters);
	else 
		trigger.action.outText("error: could not find group with name: \'" .. groupName .. "\'", 10);
	end
end;

function mission.model:addPhase(_phaseName, _description)
	mission.model.phases[_phaseName] = Factory.create(MissionPhase, { description = _description });
end; 

customEventHandler = {};

function customEventHandler:onEvent(event)
	
	-- trigger.action.outText("raised event id: " .. event.id .. ", player enter unit event id: " .. world.event.S_EVENT_PLAYER_ENTER_UNIT, 15);
	
	if (world.event.S_EVENT_DEAD == event.id) then
		local _initiator_group = event.initiator:getGroup();
		local _initiator_group_id = _initiator_group:getID();
		local _initiator_group_units = _initiator_group:getUnits();
		
		if (#_initiator_group_units < 1) then
			for _group_key, _group in pairs(mission.model.groups) do
				local _current_group = _group:getDCSWorldGroup();
				local _current_group_id = _current_group:getID(); 
			
				if (_current_group_id == _initiator_group_id) then _group:dead(event); end
			end
			
			mission.model.groups[_initiator_group:getName()] = nil;
		end
	end
	
	if (world.event.S_EVENT_BIRTH == event.id) then
		local _initiator_name = event.initiator:getName();
		
		-- trigger.action.outText('debug msg: raised event S_EVENT_BIRTH for unit ' .. _initiator_name .. ' (Id number: ' .. event.initiator:getID() .. ' )', 10);
		
		for _unit_key, _unit in pairs(mission.model.units) do
			local _current_unit_name = _unit:getUnitName();
			
			if (_initiator_name == _current_unit_name) then 
				-- trigger.action.outText('debug msg: raised event S_EVENT_BIRTH: initiator name: ' .. _initiator_name .. ' (current unit name: ' .. _current_unit_name .. ' )', 10);
				_unit.dcsWorldUnit = nil; --event.initiator; 
			end
		end
	end
	
	if (world.event.S_EVENT_TAKEOFF == event.id) then
		local _initiator_unit_id = event.initiator:getID();
		local _initiator_unit_name = event.initiator:getName();
		
		local _count_units = 0;
		
		for _unit_key, _unit in pairs(mission.model.units) do
			local _current_unit = _unit:getDCSWorldUnit();
			
			if (nil == _current_unit) then 
				_current_unit = Unit.getByName(_unit_key);
				mission.model.units[_unit_key].dcsWorldUnit = _current_unit;
			end
			
			if ((_unit_key == _initiator_unit_name) and (nil ~= _current_unit)) then _unit:takeOff(event); end
			
			_count_units = _count_units + 1;
		end
		
		-- trigger.action.outText('debug msg: raised event S_EVENT_TAKEOFF for unit. Count units: ' .. _count_units, 10);
	end
	
	if (world.event.S_EVENT_PLAYER_LEAVE_UNIT == event.id) then
		local _initiator_group = event.initiator:getGroup();
		local _initiator_group_id = _initiator_group:getID();
		
		for _group_key, _group in pairs(mission.model.groups) do
			local _current_group = _group:getDCSWorldGroup();
			local _current_group_id = _current_group:getID(); 
			
			if (_current_group_id == _initiator_group_id) then _group:playerLeaveUnit(event); end
		end
		
		local _initiator_unit_id = event.initiator:getID();
		
		for _unit_key, _unit in pairs(mission.model.units) do
			local _current_unit = _unit:getDCSWorldUnit();
			
			if (nil ~= _current_unit) then
				local _current_unit_id = _current_unit:getID();
			
				if (_current_unit_id == _initiator_unit_id) then _unit:playerLeaveUnit(event); end
			end
		end
	end
	
	if (world.event.S_EVENT_PLAYER_ENTER_UNIT == event.id) then
	
		local _initiator_group = event.initiator:getGroup();
		local _initiator_group_id = _initiator_group:getID();
		
		for _group_key, _group in pairs(mission.model.groups) do
			local _current_group = _group:getDCSWorldGroup();
			local _current_group_id = _current_group:getID(); 
			
			if (_current_group_id == _initiator_group_id) then _group:playerEnterUnit(event); end
		end
		
		local _initiator_unit_id = event.initiator:getID();
		
		for _unit_key, _unit in pairs(mission.model.units) do
			local _current_unit = _unit:getDCSWorldUnit();
			
			if (nil ~= _current_unit) then
				local _current_unit_id = _current_unit:getID();
			
				if (_current_unit_id == _initiator_unit_id) then _unit:playerEnterUnit(event); end
			end
		end	
	end

	if (world.event.S_EVENT_HIT == event.id) then
	
		local _target_group = event.target:getGroup();
		local _target_group_id = _target_group:getID();
			
		for _group_key, _group in pairs(mission.model.groups) do
			local _current_group = _group:getDCSWorldGroup();
			local _current_group_id = _current_group:getID(); 
			
			if (_current_group_id == _target_group_id) then _group:hit(event); end
		end
		
		local _target_unit_id = event.target:getID();
		
		for _unit_key, _unit in pairs(mission.model.units) do
			local _current_unit = _unit:getDCSWorldUnit();
			
			if (nil ~= _current_unit) then
				local _current_unit_id = _current_unit:getID();
			
				if (_current_unit_id == _target_unit_id) then _unit:hit(event); end
			end
		end		
	end
	
	if (world.events.S_EVENT_SHOT == event.id) then
		
		local _target_group = event.target:getGroup();
		local _target_group_id = _target_group:getID();
			
		for _group_key, _group in pairs(mission.model.groups) do
			local _current_group = _group:getDCSWorldGroup();
			local _current_group_id = _current_group:getID(); 
			
			if (_current_group_id == _target_group_id) then _group:shot(event); end
		end
		
		local _target_unit_id = event.target:getID();
		
		for _unit_key, _unit in pairs(mission.model.units) do
			local _current_unit = _unit:getDCSWorldUnit();
			
			if (nil ~= _current_unit) then
				local _current_unit_id = _current_unit:getID();
			
				if (_current_unit_id == _target_unit_id) then _unit:shot(event); end
			end
		end	
	end
end;

world.addEventHandler(customEventHandler);

trigger.action.outText('mission.lua file date: 04.11.2012 23:39', 5);